@@ -5,7 +5,7 @@ The main package for the SCons software construction utility.
 """
 
 #
-# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
+# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -27,16 +27,14 @@ The main package for the SCons software construction utility.
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #
 
-__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/__init__.py 0.96.93.D001 2006/11/06 08:31:54 knight"
+__revision__ = "/home/scons/scons/branch.0/baseline/src/engine/SCons/__init__.py 0.97.D001 2007/05/17 11:35:19 knight"
 
-__version__ = "0.96.93"
+__version__ = "0.97"
 
 __build__ = "D001"
 
 __buildsys__ = "roxbury"
 
-__date__ = "2006/11/06 08:31:54"
+__date__ = "2007/05/17 11:35:19"
 
 __developer__ = "knight"
-
-import SCons.Memoize
